/*******************************************************************************
* FILE NAME: Serial_Comm.c
*
* DESCRIPTION:
*  A simple drop in function that uses an internal timer and a normal digital output
*    to transmit serial data. Typical use is to display to a serial-capable LCD or 
*    one-way communication to an external processor. 
*  This only impacts local processing while transmission is going on, and is idle
*    while there is nothing to write.
*  This concept can be expanded to two-way asynchronous communication using a 
*    completely separate function employing an interrupt input.
*  Actual transmission can be slower than calling Write_Byte, so beware of writing too 
*    much too fast. 2400 BAUD is 240 characters per second which translates to 4+ bytes
*    per EDU slow loop or 5+ bytes per FRC slow loop.
*  This will refuse write requests whenever the output queue is full, and the user must
*    wait for space on the queue to free up.
*  
*  See README.txt included with the project files for some developmennt notes.
*
* APPLICATION NOTE:
*  To integrate this...
*	  --Setup the chosen digital pin as an output in User_Initialization() and set it below
*     --Call Initialize_Serial_Timer() from User_Initialization() (make sure the timer isn't already in use)
          Initialize_Serial_Timer(); //For serial communications on digital output
*     --Add the following to InterruptHandlerLow() for the serial transmission timing
    if (PIR3bits.TMR4IF) // timer 4 interrupt
	{
		PIR3bits.TMR4IF = 0; // clear the timer interrupt flag
		Write_Bit(); // To send the next bit of transmission data & reset the timer
	}  
*
* NOTE:
*  The slower the BAUD rate, the more forgiving the transmission will be in terms
*  of system lag.
*  The slower the BAUD rate the less impact on other processing
*  Select the User Options below
*  Things that can go wrong: TTL vs TTL-inverted, BAUD rate
*  The LCD will need a second to startup. Anything sent too soon will be lost.
*
* WRITTEN:
*  Mark McLeod
*    6/2006 - For use with Edu and Vex for training class
*
*******************************************************************************/

#include <stdio.h>
#include <timers.h>
#include <string.h>
#include "ifi_picdefs.h"
#include "ifi_aliases.h"
#include "serial_comm.h"


//******************************************************************************
//**************************** User Options ************************************
//******************************************************************************
#define TTL_LOW 1           //0=TTL, 1=Inverted TTL (determined by your choice of LCD)
#define SERIALQUEUESIZE 64  // Largest # bytes to transmit at once - usually matches size of device buffer
#define SERIAL_TX_OUTPUT rc_dig_out16  //Digital Output pin of your choice as Tx. 
                   //Make sure it's initialized as an output and set to TTL_HIGH in User_Initialize()
//******************************************************************************
//******************************************************************************

#define TTL_HIGH   !TTL_LOW
#define FALSE 	0
#define TRUE 	1

unsigned char Serial_Queue[SERIALQUEUESIZE];
unsigned char Serial_Queue_Head=0, Serial_Queue_Tail=0; //Circular queue starts empty
// Queue pointers are bytes to keep processing fast and avoid interrut handler update issues

/*******************************************************************************
* FUNCTION NAME: Write_Bit
* PURPOSE:       Handles the lowest level data transmission - one bit at a time
*                Uses an internal timer to maintain the selected BAUD rate
*                The timer interrupt causes the next bit in sequence to be written
*                or proceeds to start writing the next byte of data.
*                Sends one bit per timer interrupt until completed.
*                The Byte to transmit is taken from a global queue.
*				 This could be made simpler by using a "bit" queue rather than a "byte"
*				 queue and having Write_Byte pre-construct the entire 10-bit output. 
*				 Then just have Write_Bit pace through the bits as long as there are any
*				 eliminating the need for a state machine.
*
*                This is an interrupt handler, so it only gets called if there is a byte to send.
* NOTE:
*                When not transmitting the output pin is in IDLE or STOP_BIT
*                Bytes are transmitted as:
*                1-Start bit, 8-bits of data, 1-Stop bit, (repeat for next byte or go to IDLE)
*******************************************************************************/
unsigned char Bit_Mask[8] = {1,2,4,8,16,32,64,128};  //Mask out all but the one bit we want to send
enum{START_BIT, DATA_BITS, STOP_BIT}; //Keep track of where we are and what we are doing

void Write_Bit()
{
    static char Bit_Pointer=0; // Keeps track of each data bit we've sent
    static char State=STOP_BIT;  // Uses the enum states above, Begins at IDLE (STOP_BIT)

    switch(State)
    {
        case START_BIT:  //Start bit
                SERIAL_TX_OUTPUT = TTL_LOW;  // The Start bit is a logical "0"
                Bit_Pointer = 0; // First of the user's data to be sent
                State = DATA_BITS; // Start sending the data bits the next time we get called by the timer
                break;
        case DATA_BITS: //Transmit data byte
                SERIAL_TX_OUTPUT = Serial_Queue[Serial_Queue_Head] & Bit_Mask[Bit_Pointer++] ? TTL_HIGH : TTL_LOW;  //Send whatever the data bit is
                if (Bit_Pointer > 7)   // We've sent all the data bits
                {
                    State = STOP_BIT; // Ready for the stop bit the next time
                }
                break;
        case STOP_BIT:  //Stop bit (and IDLE)
                SERIAL_TX_OUTPUT = TTL_HIGH; // If this is the last byte to send we leave the transmit pin stopped
                State = START_BIT; // Make ready for the next byte to be sent
                if (++Serial_Queue_Head > SERIALQUEUESIZE)
                {
                    Serial_Queue_Head = 0; //Wrap queue around to the beginning (ergo circular)
                }
                if (Serial_Queue_Head == Serial_Queue_Tail) // No more bytes in the the queue to send
                {
                    T4CONbits.TMR4ON = 0; // No more data to send. Disable the interrupt from the BAUD rate timer until Write_Byte is called again.
                }
                break;
    }
}


/*******************************************************************************
* FUNCTION NAME: Write_Byte
* PURPOSE:       Organizes and queues the data to be transmitted. 
*                The Byte to transmit is simply added to a global queue.
*                Then we make sure the transmit timer is active.
* NOTES:
*		Serial_Queue_Head is changed by the interupt handler
* RETURN:
*   0 if successful, 1 if unsuccessful (queue is full)
*******************************************************************************/

char Write_Byte(unsigned char mybyte)
{
    unsigned char temp;

    temp = Serial_Queue_Tail+1;
    if (temp >= SERIALQUEUESIZE)
    {
        temp = 0;
    }
    if (temp == Serial_Queue_Head) // Is there room on the queue?
    {
        printf("FULL QUEUE ");

        return (SERIAL_UNSUCCESSFUL);
    }
    else
    {
        Serial_Queue[Serial_Queue_Tail] = mybyte;
        Serial_Queue_Tail = temp;
        T4CONbits.TMR4ON = 1; // Make sure BAUD timer is enable

        return (SERIAL_SUCCESSFUL);
    }
}


/*******************************************************************************
* FUNCTION NAME: Initialize_Serial_Timer
* PURPOSE:       Initializes the internal timer according to BAUD rate selected
*                The timer runs continuously to maintain near perfect timing, but 
*				 	does not generate interrupts until data is to be sent.
*                The transmission pin begins in an IDLE state
* NOTES:
*  It is hard to set a timer for the exact time needed, so we'll come close.
*  Timer 4 has prescales of 1:1, 1:4, or 1:16
*              post scales of 1-16
*              8-bit timer (256 ticks)
*   The PIC clock runs at 10MHz or 100ns (0.000,000,001 sec) per tick
*   2400 BAUD = 2400Hz
*       timeout  64 (0 to 64=65 ticks)  (10MHz/65=153,846.15Hz)
*       prescaler 1:16  (153,846.15/16=9615.4Hz)
*       postscalar 1:4 (9615.4Hz/4=2403.8Hz)
*       NOTE: The time will occasionally be a little stretched if the interrupt
*         occurs while other interrupts are already being processed or if interrupted by 
*         high-priority interrupt.
*
*   9600 BAUD = 9600Hz
*       prescaler 1:16
*       timeout  64 (0 to 64=65 ticks)  (625KHz/65=9615.4Hz)
*       postscalar 1:1  (9615.4Hz)
*
*******************************************************************************/
void Initialize_Serial_Timer()
{
  OpenTimer4(TIMER_INT_ON &
             T4_PS_1_16 &
             T4_POST_1_4
             );
  PR4 = 64; // Timer increments to this value then resets
  T4CONbits.TMR4ON = 0; //Disable interrupt (only use when actually writing data)
  SERIAL_TX_OUTPUT = TTL_HIGH; // Begin with the transmission pin in IDLE
  Write_Byte(0); //NULL to establish initial communication after LCD startup
}
