/*******************************************************************************
* FILE NAME: write_utilities.c
*
* DESCRIPTION:
*  This file contains simplistic serial write routines.
*  These were adapted to demonstrate serial output.
*
* USAGE:
*      define _NO_WRITES to bypass all serial write statements
*******************************************************************************/

#include <string.h>

#include "ifi_utilities.h"
#include "ifi_picdefs.h"

#include "write_utilities.h"
#include "serial_comm.h"


/******* Write utilities *******/
/*
*   These are more of an introductory course to string manipulation.
*
*      WriteString - prints fixed, unchanging strings -STRINGBUFFERMAX byte max- (rom or PIC program memory)
*      WriteRamString - prints strings constructed on-the-fly (ram or PIC data memory) -no length limit
*      WriteDecimal - print an unsigned number of any type (doesn't require type-cast)
*      WriteSignedDecimal - print a signed number of any type (doesn't require type-cast)
*
*   Example usage:
*		WriteString((char *) "gyroSum = "); //Must explicitly typecast
*       WriteSignedDecimal(gyroSum);  // For all signed integer types
*       WriteString((char *) ", heading(degrees) = ");  //Must explicitly typecast
*       WriteDecimal(heading); // For all unsigned integer types
*
*       sample output:
*            gyroSum = -734, heading(degrees) = 89
*/

#define STRINGBUFFERMAX 80  //Maximum byte length of string
   char writeBufr[STRINGBUFFERMAX]; // Prepare strings for writing out the serial output


/*******************************************************************************
* FUNCTION NAME: WriteRamString
* PURPOSE:       Write a text string to the serial output.
*                (IFI_Loader debug window or HyperTerminal)
*                Similar to PrintString, but accepts a data (ram) memory string.
* Sample Call:     char buf[20]="Hello World";
*                  WriteRamString(buf);
* ARGUMENTS:     
*     Input: bufr - pointer to data memory string.
*******************************************************************************/
void WriteRamString(char *bufr)
{
#ifndef _NO_WRITES

  int len,i;
    
  len = (int) strlen((const char *)bufr);

  for (i=0;i<len;i++)
  {
    Write_Byte(bufr[i]);
  }
#endif /* _NO_WRITES */
}

/*******************************************************************************
* FUNCTION NAME: WriteDecimal
* PURPOSE:       Write decimal data to the serial output.
* Sample Call:   WriteDecimal(seconds);
*                For any type unsigned char,int,or long
* ARGUMENTS:     
*     Input: odata - unsigned char,int, or long to be printed.
*******************************************************************************/
void WriteDecimal(unsigned long data)
{
#ifndef _NO_WRITES

  ultoa(data, writeBufr); // Convert to text string
  WriteRamString(writeBufr);

#endif /* _NO_WRITES */
}

/*******************************************************************************
* FUNCTION NAME: WriteSignedDecimal
* PURPOSE:       Write signed decimal data to the serial output.
* Sample Call:   WriteSignedDecimal(encoder);
*                For any type unsigned char,int,or long
* ARGUMENTS:     
*     Input:odata - signed char,int, or long to be printed.
*******************************************************************************/
void WriteSignedDecimal(long data)
{
#ifndef _NO_WRITES

  ltoa(data, writeBufr); // Convert to text string
  WriteRamString(writeBufr);

#endif /* _NO_WRITES */
}

/*******************************************************************************
* FUNCTION NAME: WriteString
* PURPOSE:       Prints a string to the serial port.
*                The bufr string is limited to STRINGBUFFERMAX characters (the fixed size of writeBufr)
*				 If it's longer you'll be overwriting memory...
* ARGUMENTS:     
*     bufr    word of data to be transmitted
*******************************************************************************/
void WriteString(char *bufr)
{
  static int len,I;
    
  strcpypgm2ram (writeBufr,(rom char *) bufr); /*Move from program to data memory*/
  len = (int) strlen((const char *)writeBufr);
  if (len > STRINGBUFFERMAX) len = STRINGBUFFERMAX; // truncate if it's larger than our buffer

  for (I=0;I<len;I++)
  {
     Write_Byte(writeBufr[I]);
  }
}
